@@ -5,7 +5,7 @@ import (
 	"io"
 	"strconv"
 
-	"github.com/tdewolff/parse"
+	"github.com/tdewolff/parse/v2"
 )
 
 var wsBytes = []byte(" ")
@@ -83,10 +83,11 @@ type Parser struct {
 	buf   []Token
 	level int
 
-	tt      TokenType
-	data    []byte
-	prevWS  bool
-	prevEnd bool
+	tt          TokenType
+	data        []byte
+	prevWS      bool
+	prevEnd     bool
+	prevComment bool
 }
 
 // NewParser returns a new CSS parser from an io.Reader. isInline specifies whether this is an inline style attribute.
@@ -139,12 +140,16 @@ func (p *Parser) Values() []Token {
 
 func (p *Parser) popToken(allowComment bool) (TokenType, []byte) {
 	p.prevWS = false
+	p.prevComment = false
 	tt, data := p.l.Next()
 	for tt == WhitespaceToken || tt == CommentToken {
 		if tt == WhitespaceToken {
 			p.prevWS = true
-		} else if allowComment && len(p.state) == 1 {
-			break
+		} else {
+			p.prevComment = true
+			if allowComment && len(p.state) == 1 {
+				break
+			}
 		}
 		tt, data = p.l.Next()
 	}
@@ -181,11 +186,19 @@ func (p *Parser) parseDeclarationList() GrammarType {
 	for p.tt == SemicolonToken {
 		p.tt, p.data = p.popToken(false)
 	}
+
+	// IE hack: *color:red;
+	if p.tt == DelimToken && p.data[0] == '*' {
+		tt, data := p.popToken(false)
+		p.tt = tt
+		p.data = append(p.data, data...)
+	}
+
 	if p.tt == ErrorToken {
 		return ErrorGrammar
 	} else if p.tt == AtKeywordToken {
 		return p.parseAtRule()
-	} else if p.tt == IdentToken {
+	} else if p.tt == IdentToken || p.tt == DelimToken {
 		return p.parseDeclaration()
 	} else if p.tt == CustomPropertyNameToken {
 		return p.parseCustomProperty()
@@ -194,14 +207,7 @@ func (p *Parser) parseDeclarationList() GrammarType {
 	// parse error
 	p.initBuf()
 	p.err = parse.NewErrorLexer("unexpected token in declaration", p.l.r)
-	for {
-		tt, data := p.popToken(false)
-		if (tt == SemicolonToken || tt == RightBraceToken) && p.level == 0 || tt == ErrorToken {
-			p.prevEnd = (tt == RightBraceToken)
-			return ErrorGrammar
-		}
-		p.pushBuf(tt, data)
-	}
+	return p.parseDeclarationError(p.tt, p.data, true)
 }
 
 ////////////////////////////////////////////////////////////////
@@ -353,10 +359,13 @@ func (p *Parser) parseQualifiedRuleDeclarationList() GrammarType {
 func (p *Parser) parseDeclaration() GrammarType {
 	p.initBuf()
 	parse.ToLower(p.data)
-	if tt, _ := p.popToken(false); tt != ColonToken {
+
+	tt, data := p.popToken(false)
+	if tt != ColonToken {
 		p.err = parse.NewErrorLexer("unexpected token in declaration", p.l.r)
-		return ErrorGrammar
+		return p.parseDeclarationError(tt, data, false)
 	}
+
 	skipWS := true
 	for {
 		tt, data := p.popToken(false)
@@ -370,7 +379,7 @@ func (p *Parser) parseDeclaration() GrammarType {
 		}
 		if len(data) == 1 && (data[0] == ',' || data[0] == '/' || data[0] == ':' || data[0] == '!' || data[0] == '=') {
 			skipWS = true
-		} else if p.prevWS && !skipWS {
+		} else if (p.prevWS || p.prevComment) && !skipWS {
 			p.pushBuf(WhitespaceToken, wsBytes)
 		} else {
 			skipWS = false
@@ -379,6 +388,36 @@ func (p *Parser) parseDeclaration() GrammarType {
 	}
 }
 
+func (p *Parser) parseDeclarationError(tt TokenType, data []byte, skipFirstPush bool) GrammarType {
+	first := true
+	for {
+		if first {
+			first = false
+		} else {
+			tt, data = p.popToken(false)
+		}
+		if (tt == SemicolonToken || tt == RightBraceToken) && p.level == 0 || tt == ErrorToken {
+			p.prevEnd = (tt == RightBraceToken)
+			if tt == SemicolonToken {
+				p.pushBuf(tt, data)
+			}
+			return ErrorGrammar
+		} else if tt == LeftParenthesisToken || tt == LeftBraceToken || tt == LeftBracketToken || tt == FunctionToken {
+			p.level++
+		} else if tt == RightParenthesisToken || tt == RightBraceToken || tt == RightBracketToken {
+			p.level--
+		}
+		if skipFirstPush {
+			skipFirstPush = false
+		} else {
+			if p.prevWS {
+				p.pushBuf(WhitespaceToken, wsBytes)
+			}
+			p.pushBuf(tt, data)
+		}
+	}
+}
+
 func (p *Parser) parseCustomProperty() GrammarType {
 	p.initBuf()
 	if tt, _ := p.popToken(false); tt != ColonToken {
